#include <stdio.h>
#include <stdint.h>
#include <device.h>
#include <drivers/i2c.h>
#include <string.h>
#include "ADXL345.h"

//#define ADXL_Addr 0x53
#define ADXL_Addr 0xA6
//#define ADXL_Addr 0x3A



const struct device *i2c_dev;
int err;
int i2c_read_at(uint8_t addr, uint8_t off, uint8_t *data, uint16_t len);
int i2c_write_at(uint8_t addr, uint8_t off, uint8_t *data, uint16_t len);

static int try_open_i2c_dev(void);
static int i2c_probe(uint8_t addr);

static int wait_ready(uint8_t addr);

// Local variables -----------------------------------------------------------------------------------------------------


// Local functions -----------------------------------------------------------------------------------------------------
/**
 *  @brief  Attempts to open and configure the I2C device
 *  @param  None
 *  @return 0 when successfully open, otherwise, an error code in case of failure
 */
static int try_open_i2c_dev(void) {
    int err = 0;

    if (i2c_dev) {
        return 0;
    }

    //i2c_dev = device_get_binding(I2C_DEV);
    i2c_dev = device_get_binding("I2C_1");
    if (!i2c_dev) {
        printk("I2C device driver not found.\n");
        return -ENXIO;
    }

    err = i2c_configure(i2c_dev, I2C_MODE_MASTER | I2C_SPEED_SET(I2C_SPEED_STANDARD));
    if (err) {
        printk("I2C device driver configuration failed.\n");
        i2c_dev = NULL;
        return err;
    }

    printk("I2C device driver opened and configured\n");


	//// scan for devices on the bus
	//for (uint16_t i = 4; i < 0xFF; i++) {
		//struct i2c_msg msgs[1];
		//uint8_t dst = 1;

		//msgs[0].buf = &dst;
		//msgs[0].len = 1U;
		//msgs[0].flags = I2C_MSG_WRITE | I2C_MSG_STOP;
		
		//uint8_t error = i2c_transfer(i2c_dev, &msgs[0], 1, i);
		//if (error == 0) {
			//printk("0x%2x FOUND\n", i);
		//}
		//else {
			////printk("error %d \n", error);
		//}
		//k_msleep(100);
		
	//}


    return err;
}

/**
 *  @brief  Tests the communication with an I2C device
 *  @param  uint8_t - I2C device address
 *  @return 0 when communication was successful, otherwise, an error code in case of failure
 */
static int i2c_probe(uint8_t addr) {
    int err = 0;

    // Open i2c device if not opened
    err = try_open_i2c_dev();
    if (err) {
        return err;
    }

    struct i2c_msg msg;
    uint8_t        dummy;

    msg.buf   = &dummy;
    msg.len   = sizeof(dummy);
    msg.flags = I2C_MSG_READ | I2C_MSG_STOP;

    err = i2c_transfer(i2c_dev, &msg, 1, addr >> 1);


    return err;
}

/**
 *  @brief  Helper functions to read data from an I2C device
 *  @param  uint8_t - I2C device address
 *  @param  uint8_t - I2C device address offset
 *  @param  uint8_t* - Buffer where to store the read data
 *  @param  uint16_t - Maximum length of the buffer
 *  @return Number of bytes read from the sfp
 */
int i2c_read_at(uint8_t addr, uint8_t off, uint8_t *data, uint16_t len) {
    int err = 0;

    struct i2c_msg msgs[2];

    // Setup I2C messages

    // Send the address to read from
    msgs[0].buf   = &off;
    msgs[0].len   = 1U;
    msgs[0].flags = I2C_MSG_WRITE;

    // Read from device. STOP after this
    msgs[1].buf   = data;
    msgs[1].len   = len;
    msgs[1].flags = I2C_MSG_RESTART | I2C_MSG_READ | I2C_MSG_STOP;

    err = i2c_transfer(i2c_dev, &msgs[0], 2, addr >> 1);


    return err;
}

/**
 *  @brief  Helper function to write data to an I2C device
 *  @param  uint8_t - I2C device address
 *  @param  uint8_t - I2C device address offset
 *  @param  uint8_t* - Buffer that contains the data to write
 *  @param  uint16_t - The number of bytes to write
 *  @return Number of bytes written to the I2C device
 */
int i2c_write_at(uint8_t addr, uint8_t off, uint8_t *data, uint16_t len) {
    int err = 0;

    struct i2c_msg msgs[2];

    // Setup I2C messages

    // Send the address to read from
    msgs[0].buf   = &off;
    msgs[0].len   = 1U;
    msgs[0].flags = I2C_MSG_WRITE;

    // Write to device without restart. STOP after this
    msgs[1].buf   = data;
    msgs[1].len   = len;
    msgs[1].flags = I2C_MSG_WRITE | I2C_MSG_STOP;

    err = i2c_transfer(i2c_dev, &msgs[0], 2, addr >> 1);


    return err;
}

uint8_t I2C_routine(void)
{
	try_open_i2c_dev();
	
    return 0;
}


void writeRegister(uint8_t reg, uint8_t value) 
{
	i2c_write_at(ADXL_Addr, reg, &value, 1);	
}

uint8_t readRegister(uint8_t reg) 
{
	uint8_t buff[2];
	i2c_read_at(ADXL_Addr, reg, buff, 1);
	
	return buff[0];	
}

void getData(uint16_t * data) 
{
	uint8_t buff[6];
	i2c_read_at(ADXL_Addr, ADXL345_REG_DATAX0, buff, 6);
	
	data[0] =  (((uint16_t)buff[1]) << 8) |((uint16_t)buff[0]);
	data[1] =  (((uint16_t)buff[3]) << 8) |((uint16_t)buff[2]);
	data[2] =  (((uint16_t)buff[5]) << 8) |((uint16_t)buff[4]);
}

int16_t getX(void) 
{
	uint8_t buff[2];
	i2c_read_at(ADXL_Addr, ADXL345_REG_DATAX0, buff, 2);
	
	
	uint16_t data =  (((uint16_t)buff[1]) << 8) |((uint16_t)buff[0]);
	return data;
}

int16_t getY(void)
{
	uint8_t buff[2];
	i2c_read_at(ADXL_Addr, ADXL345_REG_DATAY0, buff, 2);
	
	
	uint16_t data =  (((uint16_t)buff[1]) << 8) |((uint16_t)buff[0]);
	return data;
}

int16_t getZ(void)
{
	uint8_t buff[2];
	i2c_read_at(ADXL_Addr, ADXL345_REG_DATAZ0, buff, 2);
	
	
	uint16_t data =  (((uint16_t)buff[1]) << 8) |((uint16_t)buff[0]);
	return data;
}

bool Enable_Meas()
{
	writeRegister(ADXL345_REG_POWER_CTL, 0x08);
	return true;
}


bool Disable_Meas()
{
	writeRegister(ADXL345_REG_POWER_CTL, 0x00);
	return true;
}


void setRange(range_t range) {
  /* Read the data format register to preserve bits */
  uint8_t format = readRegister(ADXL345_REG_DATA_FORMAT);

  /* Update the data rate */
  format &= ~0x0F;
  format |= range;

  /* Make sure that the FULL-RES bit is enabled for range scaling */
  format |= 0x08;

  /* Write the register back to the IC */
  writeRegister(ADXL345_REG_DATA_FORMAT, format);

 
}


range_t getRange(void) {
  /* Read the data format register to preserve bits */
  return (range_t)(readRegister(ADXL345_REG_DATA_FORMAT) & 0x03);
}


void setDataRate(dataRate_t dataRate) {
  /* Note: The LOW_POWER bits are currently ignored and we always keep
     the device in 'normal' mode */
  writeRegister(ADXL345_REG_BW_RATE, dataRate);
}


dataRate_t getDataRate(void) {
  return (dataRate_t)(readRegister(ADXL345_REG_BW_RATE) & 0x0F);
}


float accelX() {
 
 float accx = getX() * ADXL345_MG2G_MULTIPLIER * SENSORS_GRAVITY_STANDARD;
 

  return accx;
}


float accelY() {
 
 
float accy = getY() * ADXL345_MG2G_MULTIPLIER * SENSORS_GRAVITY_STANDARD;

  return accy;
}


float accelZ() {
 

 float accz = getZ() * ADXL345_MG2G_MULTIPLIER * SENSORS_GRAVITY_STANDARD;

  return accz;
}
