#include "blef.h"
#include <bluetooth/l2cap.h>
struct bt_conn *active_conn = NULL;

volatile char ble_name[] = "APCMS";
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	//BT_DATA(BT_DATA_NAME_COMPLETE,  ble_name, 5),
	BT_DATA_BYTES(BT_DATA_UUID16_ALL,
		      BT_UUID_16_ENCODE(BT_UUID_HTS_VAL),
		      BT_UUID_16_ENCODE(BT_UUID_DIS_VAL),
		      BT_UUID_16_ENCODE(BT_UUID_BAS_VAL)),
};

static void auth_cancel(struct bt_conn *conn)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	printk("Pairing cancelled: %s\n", addr);
}

static struct bt_conn_auth_cb auth_cb_display = {
	.cancel = auth_cancel,
};

static void connected(struct bt_conn *conn, uint8_t err)
{
	if (err) {
		printk("Connection failed (err 0x%02x)\n", err);
	} else {
		active_conn = conn;
		printk("Connected\n");
	}
}

static void disconnected(struct bt_conn *conn, uint8_t reason)
{
	active_conn = NULL;
	printk("Disconnected (reason 0x%02x)\n", reason);
	// k_msleep(1000);
	// NVIC_SystemReset();
}

BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected = connected,
	.disconnected = disconnected,
};

uint8_t initBLE(void)
{
	g_BLE_INFO.udp_state = 0;
	g_BLE_INFO.udp_stream = 0;
	int err = bt_enable(NULL);
	if (err) 
	{
		printk("Bluetooth init failed (err %d)\n", err);
		return 1;
	}
	
	initASBLE();

	err = bt_le_adv_start(BT_LE_ADV_CONN_NAME, ad, ARRAY_SIZE(ad), NULL, 0);
	if (err) {
		printk("-> BLE: advertising failed to start (err %d)\n", err);
		return 2; 
	}

	
	test_l2cap_register();
	
	bt_conn_auth_cb_register(&auth_cb_display);
	
	printk("-> BLE: init done\n");
	
	return 0;
}

uint16_t getBytesAvailableBLE(void)
{
	uint16_t rval;
	if(g_BLE_INFO.rx_buff_ridx <= g_BLE_INFO.rx_buff_widx)
	{
		rval = (g_BLE_INFO.rx_buff_widx - g_BLE_INFO.rx_buff_ridx);
	}
	else
	{
		rval = (g_BLE_INFO.rx_buff_widx + (BLE_BUFFER_SIZE) - g_BLE_INFO.rx_buff_ridx);
	}
	return rval;
}

uint8_t txPktBLE(uint8_t * payload, uint16_t size)
{
	txDataASBLE(active_conn, payload, size);
	
    return 0;
}

uint8_t txSensorPktBLE(uint8_t * payload, uint16_t size)
{
	txSensorDataASBLE(active_conn, payload, size);
	
    return 0;
}

