/* Includes **************************************************************** */
#include "cmd.h"
/* Private types *********************************************************** */

/* Private constants ******************************************************* */

/* Private macros ********************************************************** */

/* Private variables ******************************************************* */
volatile  CMD_INFO g_CMD_INFO;

static uint16_t c_cmd_payload_size = CMD_PACKAGE_SIZE;

/* Private function prototypes ********************************************* */


/* Exported functions ****************************************************** */
void initCMD(void)
{
	g_CMD_INFO.state = CMD_STATE_IDLE;
	g_CMD_INFO.cmd = 0x00;
	g_CMD_INFO.size = 0x00;
	g_CMD_INFO.chunk = 0x00;
    g_CMD_INFO.system_info_timer = getSYSTIM();

#if CMD_PACKAGE_SYNCE_BYTE0_ENABLE == 1
    c_cmd_payload_size--;
#endif
#if CMD_PACKAGE_SYNCE_BYTE1_ENABLE == 1
    c_cmd_payload_size--;
#endif
#if CMD_PACKAGE_FRAGMENTATION_ENABLE == 1
    c_cmd_payload_size--;
#else
	g_CMD_INFO.chunk = 0x10;
#endif

}

void chkCMD(void)
{
    uint8_t status = getNextCMD();
    if(status != CMD_STATE_ACTIVE)
    {
        return;
    }
	
	//APP_DEBUG_LOG("-> CMD: rx-ed CMD %02X\n",g_CMD_INFO.cmd);

    uint8_t k = 0;
    for( k = 0; k < CMDAPI_SIZE; k++)
    {
        if(g_CMDAPI_INFO[k].cmd == 0x00)
        {
            break;
        }
        else if(g_CMDAPI_INFO[k].cmd == g_CMD_INFO.cmd)
        {
			//printk("-> SYS: exec %02X\n",g_CMD_INFO.cmd);
            (*g_CMDAPI_INFO[k].exeCmd)();
        }
    }
}


uint16_t getBytesAvailableCMD(void)
{
	uint16_t rval;
	if(g_cmd_rx_ridx <= g_cmd_rx_widx)
	{
		rval = (g_cmd_rx_widx - g_cmd_rx_ridx);
	}
	else
	{
		rval = (g_cmd_rx_widx + (CMD_RX_BUFFER_SIZE) - g_cmd_rx_ridx);
	}
	return rval;
}

uint8_t getNextCMD(void)
{
    if(g_CMD_INFO.state == CMD_STATE_ACTIVE)
    {
        return CMD_STATE_ACTIVE;
    }
    uint16_t size = getBytesAvailableCMD();
	//APP_DEBUG_LOG("-> CMD: SIZE %d\n",size);

    if(size < 4)
    {
        return CMD_STATE_IDLE;
    }

    uint32_t rx_ridx = g_cmd_rx_ridx;
    uint8_t crc = 0x00;
#if CMD_PACKAGE_SYNCE_BYTE0_ENABLE == 1
    if(g_cmd_rx_buffer[rx_ridx] != CMD_PACKAGE_SYNCE_BYTE0_VALUE)
    {
        printk("-> CMD: wrong SYNC0 Byte R[%02X]N[%02X]\n",g_cmd_rx_buffer[rx_ridx],CMD_PACKAGE_SYNCE_BYTE0_VALUE);
        g_cmd_rx_ridx++;
        if(g_cmd_rx_ridx >= CMD_RX_BUFFER_SIZE)
        {
            g_cmd_rx_ridx = 0x00;
        }
        return CMD_STATE_IDLE;
    }
    crc += g_cmd_rx_buffer[rx_ridx++];
    size--;
    if(rx_ridx >= CMD_RX_BUFFER_SIZE)
    {
        rx_ridx = 0x00;
    }
#endif
#if CMD_PACKAGE_SYNCE_BYTE1_ENABLE == 1
    if(g_cmd_rx_buffer[rx_ridx] != CMD_PACKAGE_SYNCE_BYTE1_VALUE)
    {
        printk("-> CMD: wrong SYNC1 Byte R[%02X]N[%02X]\n",g_cmd_rx_buffer[rx_ridx],CMD_PACKAGE_SYNCE_BYTE1_VALUE);
        g_cmd_rx_ridx++;
        if(g_cmd_rx_ridx >= CMD_RX_BUFFER_SIZE)
        {
            g_cmd_rx_ridx = 0x00;
        }
        return CMD_STATE_IDLE;
    }
    size--;
    crc += g_cmd_rx_buffer[rx_ridx++];
    if(rx_ridx >= CMD_RX_BUFFER_SIZE)
    {
        rx_ridx = 0x00;
    }
#endif
    g_CMD_INFO.cmd = g_cmd_rx_buffer[rx_ridx++];
    size--;
    if(rx_ridx >= CMD_RX_BUFFER_SIZE)
    {
        rx_ridx = 0x00;
    }
    crc += g_CMD_INFO.cmd;
#if CMD_PACKAGE_FRAGMENTATION_ENABLE == 1
    g_CMD_INFO.chunk = g_cmd_rx_buffer[rx_ridx++];
    size--;
    if(rx_ridx >= CMD_RX_BUFFER_SIZE)
    {
        rx_ridx = 0x00;
    }
    crc += g_CMD_INFO.chunk;
#endif
    g_CMD_INFO.size = g_cmd_rx_buffer[rx_ridx++];
    size--;
    if(rx_ridx >= CMD_RX_BUFFER_SIZE)
    {
        rx_ridx = 0x00;
    }
    crc += g_CMD_INFO.size;
    if((g_CMD_INFO.size + 1) > (size))
    {
        return CMD_STATE_IDLE;
    }

    uint16_t k = 0;
    uint8_t chunk = (((g_CMD_INFO.chunk >> 4) & 0x0F) - 1);
    for( k = 0; k < g_CMD_INFO.size; k++ )
    {
        g_CMD_INFO.payload[chunk][k] = g_cmd_rx_buffer[rx_ridx++];
        if(rx_ridx >= CMD_RX_BUFFER_SIZE)
        {
            rx_ridx = 0x00;
        }
        crc += g_CMD_INFO.payload[chunk][k];
    }

    crc = ~(crc) + 1;
    g_CMD_INFO.crc = g_cmd_rx_buffer[rx_ridx++];
    if(rx_ridx >= CMD_RX_BUFFER_SIZE)
    {
        rx_ridx = 0x00;
    }
    if(g_CMD_INFO.crc != crc)
    {
        printk("-> CMD: wrong CRC received R[%02X]N[%02X] CMD[%02X] SIZE %d\n",g_CMD_INFO.crc,crc,g_CMD_INFO.cmd,g_CMD_INFO.size );
		{
			for(k=0;k<g_CMD_INFO.size;k++)
			{
				printk("%02X ", g_CMD_INFO.payload[chunk][k]);
			}
			printk("\n");
		}
        g_cmd_rx_ridx = rx_ridx;
        return CMD_STATE_CRC_ERROR;
    }
    g_cmd_rx_ridx = rx_ridx;
    return CMD_STATE_ACTIVE;
}

void txCMD(uint8_t cmd,uint8_t chunk,uint8_t * payload,uint8_t size)
{
    if(size > c_cmd_payload_size)
    {
        printk("-> CMD: Tx cmd failed [%d/%d]\n",size,c_cmd_payload_size);
        return;
    }
    uint8_t k = 0;
    uint8_t n = 0;
    uint8_t crc = 0x00;
#if CMD_PACKAGE_SYNCE_BYTE0_ENABLE == 1
    g_cmd_rx_buffer[k] = CMD_PACKAGE_SYNCE_BYTE0_VALUE;
    crc += g_cmd_rx_buffer[k];
    k++;
#endif
#if CMD_PACKAGE_SYNCE_BYTE1_ENABLE == 1
    g_cmd_rx_buffer[k] = CMD_PACKAGE_SYNCE_BYTE1_VALUE;
    crc += g_cmd_rx_buffer[k];
    k++;
#endif
    g_cmd_rx_buffer[k] = cmd;
    crc += g_cmd_rx_buffer[k];
    k++;
#if CMD_PACKAGE_FRAGMENTATION_ENABLE == 1
    g_cmd_rx_buffer[k] = chunk;
    crc += g_cmd_rx_buffer[k];
    k++;
#endif
    g_cmd_rx_buffer[k] = size;
    crc += g_cmd_rx_buffer[k];
    k++;
    for( n = 0; n < size; n++ )
    {
        g_cmd_rx_buffer[k]  = payload[n];
        crc += g_cmd_rx_buffer[k];
        k++;
    }
    crc = (~crc) + 1;
    g_cmd_rx_buffer[k] = crc;
    k++;
    txPktCMD((uint8_t *)g_cmd_rx_buffer,k);
}

/* Private functions ******************************************************* */

/* ***************************** END OF FILE ******************************* */


