#include "pdm.h"
#include "audio.h"
#include "sensor.h"

volatile uint8_t flag =0;

typedef enum
{
    PDM_STATUS_UNINITIALIZED, 			/**< Uninitialized. */
    PDM_STATUS_INITIALIZED, 			/**< Initialized but powered off. */
    PDM_STATUS_POWERED_ON
} PDM_STATUS;

typedef enum
{
    PDM_STATE_IDLE,
    PDM_STATE_RUNNING,
    PDM_STATE_STARTING,
    PDM_STATE_STOPPING
} PDM_STATE;

PDM_INFO g_PDM_INFO;
struct k_poll_signal signal_eram_action;

void nrfx_pdm1_irq_handler(void * p_event, void * p_context)
{
	
    if(NRF_PDM0->EVENTS_STARTED)
    {
        NRF_PDM0->EVENTS_STARTED = 0;

        uint8_t finished_buffer = g_PDM_INFO.active_buffer;

        // Check if the next buffer was set before.
        uint8_t next_buffer = (~g_PDM_INFO.active_buffer) & 0x01;
        if (g_PDM_INFO.buff_address[next_buffer] || g_PDM_INFO.state == PDM_STATE_STARTING)
        {
            g_PDM_INFO.error = 0;

            // Release the full buffer if ready and request the next one.
            if (g_PDM_INFO.state == PDM_STATE_STARTING)
            {
                g_PDM_INFO.state = PDM_STATE_RUNNING;
            }
            else
            {
                g_PDM_INFO.buff_address[finished_buffer] = 0;
                g_PDM_INFO.active_buffer = next_buffer;
            }
            g_PDM_INFO.event_handler(1);
        }
        else
        {
            // Do not request the new buffer as it was already done.
            if (g_PDM_INFO.error == 0)
            {
				printk("-> PDM: error?!\n");
                g_PDM_INFO.error = 1;
                g_PDM_INFO.event_handler(0);
            }
        }

        if (g_PDM_INFO.state == PDM_STATE_STARTING)
        {
            g_PDM_INFO.state = PDM_STATE_RUNNING;
        }
    }
    else if(NRF_PDM0->EVENTS_STOPPED)
    {
        NRF_PDM0->EVENTS_STOPPED = 0;
        NRF_PDM0->ENABLE = 0;
        g_PDM_INFO.state = PDM_STATE_IDLE;

        // Release the buffers.
        if (g_PDM_INFO.buff_address[g_PDM_INFO.active_buffer])
        {
            g_PDM_INFO.buff_address[g_PDM_INFO.active_buffer] = 0;
            g_PDM_INFO.event_handler(0);
        }

        uint8_t second_buffer = (~g_PDM_INFO.active_buffer) & 0x01;
        if (g_PDM_INFO.buff_address[second_buffer])
        {
            g_PDM_INFO.buff_address[second_buffer] = 0;
            g_PDM_INFO.event_handler(0);
        }
        g_PDM_INFO.active_buffer = 0;
    }

    if (g_PDM_INFO.irq_buff_request)
    {
        g_PDM_INFO.irq_buff_request = 0;
        g_PDM_INFO.event_handler(1);
    }
}
		    
static void drv_pdm_hand(uint8_t buffer_requested)
{
	if(buffer_requested)
    {
		//int16_t * ptr;
		if(g_PDM_INFO.bflag == 0)
		{
			setBufferPDM((int16_t * )g_AUDIO_INFO.buff0, PDM_DMA_BUFF_SIZE);
			//ptr = (int16_t * )g_AUDIO_INFO.buff1;
		}
		else
		{
			setBufferPDM((int16_t * )g_AUDIO_INFO.buff1, PDM_DMA_BUFF_SIZE);
			//ptr = (int16_t * )g_AUDIO_INFO.buff0;
		}
		
		g_PDM_INFO.bflag ^= 0x01;
		if(g_PDM_INFO.skip_initial >= 4)
		{
			// raise a signal for main loop to copy data to ERAM
			k_poll_signal_raise(&signal_eram_action, 0);
		}
		else
		{
			g_PDM_INFO.skip_initial++;
		}
		        
        if(g_AUDIO_INFO.widx > (g_AUDIO_INFO.cap_size+(PDM_DMA_BUFF_SIZE)))
        {
			g_AUDIO_INFO.widx = g_AUDIO_INFO.cap_size;
			stopPDM();
			//endSENSOR();
			g_AUDIO_INFO.state = (AUDIO_STATE_CAPTURE_COMPLETED);
			printk("-> AUD: capture completed %d samples TRG: %d\n",g_AUDIO_INFO.cap_size,g_PDM_INFO.trg_cnt);
			
			uint8_t pkt[2];
			pkt[0] = AUDIO_STREAMING_STATUS;
			pkt[1] = AUDIO_STREAMING_COMPLETE;
			txSensorPktBLE(pkt, 2);
			
		}
		
		g_PDM_INFO.trg_cnt++;
    }
}

void initPDM(void)
{
    g_PDM_INFO.buff_address[0] = 0;
    g_PDM_INFO.buff_address[1] = 0;
    g_PDM_INFO.active_buffer = 0;
    g_PDM_INFO.error = 0;
    g_PDM_INFO.event_handler = drv_pdm_hand;
    g_PDM_INFO.state = PDM_STATE_IDLE;

	// set 1280kHz frequency
    NRF_PDM0->PDMCLKCTRL = 0x0A000000;
    // set factor of 80 to get 16kHz sampling frequency
	NRF_PDM0->RATIO = 1;
    // set 1032 frequency
    //NRF_PDM0->PDMCLKCTRL = 0x08400000;
    // set factor of 64 to get 16kHz sampling frequency
	//NRF_PDM0->RATIO = 0;
    
	// set mono mode and falling edge sampling on PDM CLK
    NRF_PDM0->MODE = 0x00000001;
    // set 0dB gain adjustment
    NRF_PDM0->GAINL = 0x00000028;
    NRF_PDM0->GAINR = 0x00000028;

	NRF_P0->PIN_CNF[PDM_CLK_PIN] = 0x00000001;
    //NRF_P0->OUTCLR = (1<<PDM_CLK_PIN);
	NRF_P0->PIN_CNF[PDM_DIN_PIN] = 0x00000000;
    NRF_PDM0->PSEL.CLK = PDM_CLK_PIN;
    NRF_PDM0->PSEL.DIN = PDM_DIN_PIN;
	NRF_PDM0->MCLKCONFIG = 0;
	
	NRF_PDM0->EVENTS_STARTED = 0;
	NRF_PDM0->EVENTS_END = 0;
	NRF_PDM0->EVENTS_STOPPED = 0;
	// enable started and stopped events interrupt
	NRF_PDM0->INTENSET = 0x00000003;


    NVIC_SetPriority(PDM0_IRQn, 3);
    NVIC_ClearPendingIRQ(PDM0_IRQn);
    NVIC_EnableIRQ(PDM0_IRQn);
    g_PDM_INFO.status = PDM_STATUS_INITIALIZED;
    
    IRQ_CONNECT(PDM0_IRQn, 0, nrfx_pdm1_irq_handler, 0, 0);
}

void deinitPDM(void)
{
    NRF_PDM0->ENABLE = 0;
    NRF_PDM0->PSEL.CLK = 0;
    NRF_PDM0->PSEL.DIN = 0;
    g_PDM_INFO.status = PDM_STATUS_UNINITIALIZED;
}

void triggerPDM(void)
{
    g_PDM_INFO.status = PDM_STATUS_POWERED_ON;
    NRF_PDM0->ENABLE = 1;
    NRF_PDM0->EVENTS_STARTED = 0;
    NRF_PDM0->TASKS_START = 1;
}

void requestBufferPDM(void)
{
    g_PDM_INFO.irq_buff_request = 1;
    NVIC_SetPendingIRQ(PDM0_IRQn);
}

void startPDM(void)
{
	g_PDM_INFO.skip_initial = 0;
	g_PDM_INFO.bflag = 0;
	g_PDM_INFO.trg_cnt = 0;
    g_PDM_INFO.state = PDM_STATE_STARTING;
    requestBufferPDM();
    
    //k_timer_start(&accel_timer, K_MSEC(10), K_NO_WAIT);
}

void setBufferPDM(int16_t * buffer, uint16_t buffer_length)
{
    NVIC_DisableIRQ(PDM0_IRQn);

    uint8_t next_buffer = (~g_PDM_INFO.active_buffer) & 0x01;
    if (g_PDM_INFO.state == PDM_STATE_STARTING)
    {
        next_buffer = 0;
    }

    if (g_PDM_INFO.buff_address[next_buffer] == 0x00)
    {
        g_PDM_INFO.buff_address[next_buffer] = buffer;
        g_PDM_INFO.buff_length[next_buffer] = buffer_length;
        NRF_PDM0->SAMPLE.PTR = (uint32_t)buffer;
        NRF_PDM0->SAMPLE.MAXCNT = buffer_length;
        if (g_PDM_INFO.status != PDM_STATUS_POWERED_ON)
        {
			triggerPDM();
        }
    }
    
    NVIC_EnableIRQ(PDM0_IRQn);
}

void stopPDM(void)
{
    if (g_PDM_INFO.state != PDM_STATE_RUNNING)
    {
        if (g_PDM_INFO.state == PDM_STATE_IDLE || g_PDM_INFO.state == PDM_STATE_STARTING)
        {
            NRF_PDM0->ENABLE = 0;
            g_PDM_INFO.state = PDM_STATE_IDLE;
            return;
        }
    }
    g_PDM_INFO.status = PDM_STATUS_INITIALIZED;
    g_PDM_INFO.state = PDM_STATE_STOPPING;

	NRF_PDM0->TASKS_STOP = 1;
}
